/*------------------------------------------------------------------------------
 bin/djv_view/view.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_VIEW_H
#define DJV_VIEW_VIEW_H

#include <djv_ui/image.h>
#include "base.h"

namespace view {

// Grids.

enum GRID { GRID_NONE, GRID_1x1, GRID_10x10, GRID_100x100 };

// Heads up display (HUD) information.

class Hud_Info : public Info {
public:
  
  Hud_Info() { init(); }
  Hud_Info(const Info & in) : Info(in) { init(); }

  void init()
  {
    playback_frame = 0;
    playback_frame_start = 0;
    playback_frame_end = 0;
    playback_speed_real = 0.0;
    playback_dropped_frames = false;
  }

  int playback_frame, playback_frame_start, playback_frame_end;
  image::time::Speed playback_speed;
  double playback_speed_real;
  bool playback_dropped_frames;
};

// View widget.

class Widget : public ui::image::Widget {
public:

  Widget();

  // View.
  
  void zoom_in(const Vec2i &);
  void zoom_in();
  void zoom_out(const Vec2i &);
  void zoom_out();
  
  // Overlays.
  
  void grid(GRID);
  void grid_color(const image::Color &);

  void hud(bool);
  void hud_info(const Hud_Info &);
  void hud_foreground(const image::Color &);
  void hud_background(const image::Color &);
  
  // Widget.
  
  int handle(int);
  void draw();
  
  ui::Signal<const Vec2i &> pick_sig;
  
  // Layout.
  
  void dirty();

private:

  void draw_grid();
  void draw_hud();
  static void draw_text(const Str &, const Vec2i & position,
    const image::Color & foreground, const image::Color & background);
  
  Vec2i _view_tmp;
  GRID _grid;
  image::Color _grid_color;
  bool _hud;
  Hud_Info _hud_info;
  image::Color _hud_foreground, _hud_background;
  bool _inside;
  Vec2i _mouse, _mouse_td;
};

// Labels.

const List<Str> & grid_label();

}

#endif

