/*------------------------------------------------------------------------------
 bin/djv_view/window_factory.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "window.h"
#include <djv_ui/app.h>

namespace window {

Factory::~Factory()
{
  DJV_DBG_OBJ("window::Factory::~Factory");
  
  for (uint i = 0; i < _list.size(); ++i)
    _list[i]->widget_del();
}

Widget * Factory::add(Widget * in)
{
  DJV_DBG_OBJ("window::Factory::add");
  DJV_DBG_PRN("in = " << int(in));
  
  Widget * window = new Widget(in);
  _list += window;
  return window;
}

void Factory::del(Widget * in)
{
  DJV_DBG_OBJ("window::Factory::del");
  DJV_DBG_PRN("in = " << int(in));
  
  in->widget_del();
  _list.erase(std::find(_list.begin(), _list.end(), in));  

  // Exit application if no windows are left.
  
  if (! _list.size()) ui::app::app()->exit();
}

namespace { std::auto_ptr<Factory> _factory; }

Factory * factory()
{
  if (! _factory.get()) _factory = std::auto_ptr<Factory>(new Factory);
  return _factory.get();
}

void factory_del() { _factory.reset(); }

}

