/*------------------------------------------------------------------------------
 lib/djv_base/box2_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv {
namespace box {

#define _BOX2_INIT \
  x(Box<T, 2>::position.e[0]), \
  y(Box<T, 2>::position.e[1]), \
  w(Box<T, 2>::size.e[0]), \
  h(Box<T, 2>::size.e[1])

template<typename T>
inline Box2<T>::Box2() : _BOX2_INIT {}

template<typename T>
inline Box2<T>::Box2(const Box2<T> & in) : Box<T, 2>(in), _BOX2_INIT {}

template<typename T>
inline Box2<T>::Box2(const Box<T, 2> & in) : Box<T, 2>(in), _BOX2_INIT {}

template<typename T>
inline Box2<T>::Box2(T x, T y, T w, T h) :
  Box<T, 2>(Vec2<T>(x, y), Vec2<T>(w, h)), _BOX2_INIT {}

template<typename T>
inline Box2<T>::Box2(T w, T h) : Box<T, 2>(Vec2<T>(w, h)), _BOX2_INIT {}

template<typename T>
inline Box2<T>::Box2(const Vec2<T> & position, const Vec2<T> & size) :
  Box<T, 2>(position, size), _BOX2_INIT {}

template<typename T>
inline Box2<T>::Box2(const Vec2<T> & size) : Box<T, 2>(size), _BOX2_INIT {}

template<typename T>
inline void Box2<T>::x1(const T & in) { w = in - x; }
template<typename T>
inline void Box2<T>::y1(const T & in) { h = in - y; }

template<>
inline void Box2<int>::x1(const int & in) { w = in - x + 1; }
template<>
inline void Box2<int>::y1(const int & in) { h = in - y + 1; }

template<typename T>
inline T Box2<T>::x1() const { return x + w; }
template<typename T>
inline T Box2<T>::y1() const { return y + h; }

template<>
inline int Box2<int>::x1() const { return x + w - 1; }
template<>
inline int Box2<int>::y1() const { return y + h - 1; }

template<typename T>
inline void Box2<T>::position1(const Vec2<T> & in)
{
  w = in.x - x;
  h = in.y - y;
}

template<typename T>
inline Vec2<T> Box2<T>::position1() const
{
  return Vec2<T>(x + w, y + h);
}

template<>
inline void Box2<int>::position1(const Vec2<int> & in)
{
  w = in.x - x + 1;
  h = in.y - y + 1;
}

template<>
inline Vec2<int> Box2<int>::position1() const
{
  return Vec2<int>(x + w - 1, y + h - 1);
}

}
}

