/*------------------------------------------------------------------------------
 lib/djv_base/dbg.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "dbg.h"
#include <djv_base/str.h>
#include <stdio.h>
#include <stdlib.h>

namespace djv {
namespace dbg {

void _djv_assert(const char * file, int line)
{
  printf("ASSERT: File \"%s\", line %d.\n", file, line);

  abort();
}

// XXX

namespace { int indent = 0; }

void Dbg::_Dbg(const Str & in)
{
  *this << BEGIN << in << END;
  *this << BEGIN << "{" << END;

  indent += 2;
}

Dbg::Dbg(const Str & in) { _Dbg(in); }
Dbg::Dbg(const Str & prefix, const Str & in) { _Dbg(prefix + ": " + in); }

Dbg::~Dbg()
{
  _timer.check();

  indent -= 2;
  
  *this << BEGIN << "} (" << _timer.seconds() << " seconds, " << _timer.fps()
    << " fps)" << END;
}

Dbg & Dbg::operator << (LINE in)
{
  switch (in) {
    case BEGIN: _tmp = "dbg " + Str(indent, ' '); break;
    case END: printf("%s\n", _tmp.c_str()); break;
  }
  return *this;
}

Dbg & Dbg::operator << (const Str & in) { add(in); return *this; }
Dbg & Dbg::operator << (const char * in) { add(in); return *this; }

void Dbg::add(const Str & in) { _tmp += in; }

Dbg & operator << (Dbg & dbg, bool in)
  { return dbg << (in ? "true" : "false"); }
Dbg & operator << (Dbg & dbg, int in)
  { return dbg << str::int_to_str(in); }
Dbg & operator << (Dbg & dbg, uint in)
  { return dbg << str::int_to_str(int(in)); }
Dbg & operator << (Dbg & dbg, double in)
  { return dbg << str::float_to_str(in); }
Dbg & operator << (Dbg & dbg, const List<Str> & in)
  { return dbg << str::join(in, " "); }

namespace {

Str bits(uint32_t in, int size)
{
  Str out;
  for (int i = 0; i < size; ++i)
    out += ((in >> i) & 1) ? '1' : '0';
  return out;
}

}

Str u8(uint8_t in) { return bits(in, 8); }
Str u16(uint16_t in) { return bits(in, 16); }
Str u32(uint32_t in) { return bits(in, 32); }

}
}
