/*------------------------------------------------------------------------------
 lib/djv_base/file_split.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_base/file_split.cxx"

#include "file.h"

namespace djv {
namespace file {

// XXX

#define _PATH_SEPARATOR(IN) \
    ((IN) == path_separator_list[0] || \
     (IN) == path_separator_list[1])

void File::split(const Str & in, Str * path, Str * base, Str * number,
  Str * extension)
{
  DJV_DBG_OBJ2(_DBG, "split");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("size = " << in.size());
  
  const int size = int(in.size());
  
  path->clear();
  base->clear();
  number->clear();
  extension->clear();
  if (! size) return;

  // Extension.

  int i = size - 1;
  int tmp = i;
  for (; in[i] != '.' && ! _PATH_SEPARATOR(in[i]) && i > 0; --i) ;
  if (i > 0 && '.' == in[i]) {
    *extension = Str(in, i, tmp - i + 1);
    --i;
  } else
    i = size - 1;
  DJV_DBG_PRN("extension = " << *extension);

  // Number.

  if (i >= 0 && str::is_digit(in[i])) {
    tmp = i;
    for (; i > 0 && (str::is_digit(in[i - 1]) || '-' == in[i - 1]); --i) ;
    *number = Str(in, i, tmp - i + 1);
    --i;
  }
  DJV_DBG_PRN("number = " << *number);

  // Base.
  
  if (i >= 0 && ! _PATH_SEPARATOR(in[i])) {
    tmp = i;
    for (; i > 0 && ! _PATH_SEPARATOR(in[i - 1]); --i) ;
    *base = Str(in, i, tmp - i + 1);
    --i;
  }
  DJV_DBG_PRN("base = " << *base);

  // Path.

  if (i >= 0)
    *path = Str(in, 0, i + 1);
  DJV_DBG_PRN("path = " << *path);
}

}
}
