/*------------------------------------------------------------------------------
 lib/djv_base/mtx_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv {
namespace mtx {

template<typename T, int E>
inline void Mtx<T, E>::set(const Mtx<T, E> & in)
{
  for (int i = 0; i < E * E; ++i)
    e[i] = in.e[i];
}

template<typename T, int E>
inline void Mtx<T, E>::set(const T * in)
{
  for (int i = 0; i < E * E; ++i)
    e[i] = in[i];
}

template<typename T, int E>
inline void Mtx<T, E>::identity()
{
  for (int i = 0; i < E; ++i)
    for (int j = 0; j < E; ++j)
      e[i * E + j] = (i == j) ? 1.0 : 0.0;
}

template<typename T, int E>
inline void Mtx<T, E>::zero()
{
  for (int i = 0; i < E * E; ++i)
    e[i] = 0.0;
}

template<typename T>
inline Mtx3<T>::Mtx3(T _0, T _1, T _2, T _3, T _4, T _5, T _6, T _7, T _8)
{
  set(_0, _1, _2, _3, _4, _5, _6, _7, _8);
}

template<typename T>
inline void Mtx3<T>::set(T _0, T _1, T _2, T _3, T _4, T _5, T _6, T _7, T _8)
{
  T * e = Mtx<T, 3>::e;
  e[0] = _0; e[1] = _1; e[2] = _2;
  e[3] = _3; e[4] = _4; e[5] = _5;
  e[6] = _6; e[7] = _7; e[8] = _8;
}

template<typename T>
inline Mtx4<T>::Mtx4(T _0, T _1, T _2, T _3, T _4, T _5, T _6, T _7, T _8,
  T _9, T _10, T _11, T _12, T _13, T _14, T _15
) {
  set(_0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15);
}

template<typename T>
inline void Mtx4<T>::set(T _0, T _1, T _2, T _3, T _4, T _5, T _6, T _7, T _8,
  T _9, T _10, T _11, T _12, T _13, T _14, T _15
) {
  T * _e = Mtx<T, 4>::e;
  _e[0] = _0; _e[1] = _1; _e[2] = _2; _e[3] = _3;
  _e[4] = _4; _e[5] = _5; _e[6] = _6; _e[7] = _7;
  _e[8] = _8; _e[9] = _9; _e[10] = _10; _e[11] = _11;
  _e[12] = _12; _e[13] = _13; _e[14] = _14; _e[15] = _15;
}

}
}

