/*------------------------------------------------------------------------------
 lib/djv_base/mtx_fnc_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/math.h>

namespace djv {
namespace mtx {

template<typename T, int E>
inline Mtx<T, E> transpose(const Mtx<T, E> & in)
{
  Mtx<T, E> out;
  for (int i = 0; i < E; ++i)
    for (int j = 0; j < E; ++j)
      out.e[j * E + i] = in.e[i * E + j];
  return out;
}

template<typename T, int E>
inline Mtx<T, E> operator * (const Mtx<T, E> & a, const Mtx<T, E> & b)
{
  Mtx<T, E> out;
  out.zero();
  for (int i = 0; i < E; ++i)
    for (int j = 0; j < E; ++j)
      for (int k = 0; k < E; ++k)
        out.e[i * E + j] += a.e[i * E + k] * b.e[k * E + j];
  return out;
}

template<typename T, int E>
inline bool operator == (const Mtx<T, E> & a, const Mtx<T, E> & b)
{
  for (int i = 0; i < E * E; ++i)
    if (a.e[i] != b.e[i]) return false;
  return true;
}

template<typename T, int E>
inline bool operator != (const Mtx<T, E> & a, const Mtx<T, E> & b)
  { return ! (a == b); }

template<typename T, int E>
inline bool serialize(List<Str> * in, Mtx<T, E> * out)
{
  for (int i = 0; i < E * E; ++i)
    if (! str::serialize(in, &out->e[i])) return false;
  return true;
}

template<typename T, int E>
inline List<Str> serialize(const Mtx<T, E> & in)
{
  List<Str> out;
  for (int i = 0; i < E * E; ++i)
    out += str::serialize(in.e[i]);
  return out;
}

template<typename T, int E>
inline Dbg & operator << (Dbg & dbg, const Mtx<T, E> & in)
{
  dbg << "\n";
  for (int i = 0; i < E; ++i) {
    for (int j = 0; j < E; ++j) {
      dbg << in.e[i * E + j];
      if (j < E - 1) dbg << " ";
    }
    if (i < E - 1) dbg << "\n";
  }
  return dbg;
}

}
}

