/*------------------------------------------------------------------------------
 lib/djv_image/color_fnc.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "color.h"

namespace djv {
namespace image {
namespace color {
using namespace pixel;

void convert(const Color & in, Color * out)
{
  pixel::convert(in.data(), in.pixel(), out->data(), out->pixel());
}

bool serialize(List<Str> * in, Color * out)
{
  PIXEL pixel;
  if (! pixel::serialize(in, &pixel)) return false;
  out->set(pixel);
  
  const int channels = pixel::channels(pixel);
  for (int c = 0; c < channels; ++c)
    switch (pixel::type(pixel)) {
      case U8: {
        int value = 0;
        if (! str::serialize(in, &value)) return false;
        out->set_u8(math::clamp(value, 0, u8_max), c);
      } break;
      case U10: {
        int value = 0;
        if (! str::serialize(in, &value)) return false;
        out->set_u10(math::clamp(value, 0, u10_max), c);
      } break;
      case U16: {
        int value = 0;
        if (! str::serialize(in, &value)) return false;
        out->set_u16(math::clamp(value, 0, pixel::u16_max), c);
      } break;
      case F16: {
        double value = 0.0;
        if (! str::serialize(in, &value)) return false;
        out->set_f16(value, c);
      } break;
      case F32: {
        double value = 0.0;
        if (! str::serialize(in, &value)) return false;
        out->set_f32(value, c);
      } break;
    }
    
  return true;
}

List<Str> serialize(const Color & in)
{
  List<Str> out = List<Str>() << pixel::serialize(in.pixel());
  
  const int channels = pixel::channels(in.pixel());
  for (int c = 0; c < channels; ++c)
    switch (pixel::type(in.pixel())) {
      case U8: out << str::serialize(in.get_u8(c)); break;
      case U10: out << str::serialize(in.get_u10(c)); break;
      case U16: out << str::serialize(in.get_u16(c)); break;
      case F16: out << str::serialize(in.get_f16(c)); break;
      case F32: out << str::serialize(in.get_f32(c)); break;
    }
  
  return out;
}

bool operator == (const Color & a, const Color & b)
{
  if (a.pixel() != b.pixel()) return false;
  return mem::compare(a.data(), b.data(), pixel::bytes(a.pixel())) == 0;
}

bool operator != (const Color & a, const Color & b) { return ! (a == b); }

Str label(const Color & in)
{
  return str::join(serialize(in), " ");
}

Dbg & operator << (Dbg & dbg, const Color & in)
  { return dbg << serialize(in); }

}
}
}
