/*------------------------------------------------------------------------------
 lib/djv_image/data.h
 
 Image Data
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_DATA_H
#define DJV_IMAGE_DATA_H

#include <djv_base/vec.h>
#include <djv_image/pixel.h>

namespace djv {
namespace image {
namespace data {

class Info {
public:

  Info() :
    proxy(0),
    pixel(pixel::PIXEL(0)), bgr(false),
    mirror_x(false), mirror_y(false),
    align(1), endian(mem::endian_)
  {}
  
  Info(const Vec2i & size, pixel::PIXEL pixel) :
    size(size), proxy(0),
    pixel(pixel), bgr(false),
    mirror_x(false), mirror_y(false),
    align(1), endian(mem::endian_)
  {}

  Vec2i size;
  int proxy;
  pixel::PIXEL pixel;
  bool bgr;
  bool mirror_x, mirror_y;
  int align;
  mem::ENDIAN endian;
};

class Data {
public:

  Data();
  Data(const Data &);
  Data(const Info &, const uint8_t * = 0);

  Data & operator = (const Data &);

  // Value.
  
  void set(const Data &);
  void set(const Info &, const uint8_t * = 0);
  
  const Info & info() const { return _info; }
  const Vec2i & size() const { return _info.size; }
  int proxy() const { return _info.proxy; }
  int w() const { return _info.size.e[0]; }
  int h() const { return _info.size.e[1]; }
  pixel::PIXEL pixel() const { return _info.pixel; }
  int channels() const { return _channels; }
  
  // Functions.
  
  void zero();
  
  // Data.
  
  uint8_t * data() { return _data(); }
  const uint8_t * data() const { return _p; }

  uint8_t * data(int x, int y)
    { return _data() + (y * _info.size.e[0] + x) * _pixel_bytes; }
  const uint8_t * data(int x, int y) const
    { return _p + (y * _info.size.e[0] + x) * _pixel_bytes; }
  
  size_t pixel_bytes() const { return _pixel_bytes; }
  size_t scanline_bytes() const { return _scanline_bytes; }
  size_t data_bytes() const { return _data_bytes; }

private:

  void init();
  void alloc();
  void del();
  
  Info _info;
  int _channels;
  mem::Buffer<uint8_t> _data;
  size_t _pixel_bytes, _scanline_bytes, _data_bytes;
  const uint8_t * _p;
};

// Functions.

size_t scanline_bytes(const Info &);
size_t data_bytes(const Info &);

void proxy_scale(const Data &, Data *, int proxy);
int proxy_scale(int proxy);
Vec2i proxy_scale(const Vec2i &, int proxy);

void planar_interleave(const Data &, Data *, int proxy = 0);
void planar_deinterleave(const Data &, Data *);

// Comparison.

bool operator == (const Info &, const Info &);
bool operator != (const Info &, const Info &);

// Debugging.

Dbg & operator << (Dbg &, const Info &);
Dbg & operator << (Dbg &, const Data &);

}
}
}

#endif
