/*------------------------------------------------------------------------------
 lib/djv_image/gl.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "gl.h"
#include <djv_base/math.h>

namespace djv {
namespace image {
namespace gl {

void ortho(int width, int height, double min, double max)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0.0, width, 0.0, height, min, max);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

GLenum format(pixel::PIXEL in, bool bgr)
{
  GLenum out = 0;
  switch (in) {
    case pixel::L_U8:
    case pixel::L_U16:
    case pixel::L_F16:
    case pixel::L_F32: out = GL_LUMINANCE; break;
    case pixel::LA_U8:
    case pixel::LA_U16:
    case pixel::LA_F16:
    case pixel::LA_F32: out = GL_LUMINANCE_ALPHA; break;
    case pixel::RGB_U8:
    case pixel::RGB_U16:
    case pixel::RGB_F16:
    case pixel::RGB_F32: out = bgr ? GL_BGR : GL_RGB; break;
    case pixel::RGB_U10: out = bgr ? GL_BGRA : GL_RGBA; break;
    case pixel::RGBA_U8:
    case pixel::RGBA_U16:
    case pixel::RGBA_F16: 
    case pixel::RGBA_F32: out = bgr ? GL_BGRA : GL_RGBA; break;
  }
  return out;
}

GLenum type(pixel::PIXEL in)
{
  GLenum out = 0;
  switch (in) {
    case pixel::L_U8:
    case pixel::LA_U8:
    case pixel::RGB_U8:
    case pixel::RGBA_U8: out = GL_UNSIGNED_BYTE; break;
    case pixel::RGB_U10: out = GL_UNSIGNED_INT_10_10_10_2; break;
    case pixel::L_U16:
    case pixel::LA_U16:
    case pixel::RGB_U16:
    case pixel::RGBA_U16: out = GL_UNSIGNED_SHORT; break;
    case pixel::L_F16:
    case pixel::LA_F16:
    case pixel::RGB_F16:
    case pixel::RGBA_F16:
      out = GLEW_ARB_half_float_pixel ? GL_HALF_FLOAT_ARB : 0; break;
    case pixel::L_F32:
    case pixel::LA_F32:
    case pixel::RGB_F32:
    case pixel::RGBA_F32: out = GL_FLOAT; break;
  }
  return out;
}

}
}
}

