/*------------------------------------------------------------------------------
 lib/djv_image/gl_data.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/gl_data.cxx"

#include "_gl_data.h"

namespace djv {
namespace image {
namespace gl {
namespace data {

Display::Levels & Display::Levels::operator = (const Levels & _in)
{
  in[0] = _in.in[0];
  in[1] = _in.in[1];
  gamma = _in.gamma;
  out[0] = _in.out[0];
  out[1] = _in.out[1];
  return *this;
}

Data::Data(const image::Data * in) :
  _data(0),
  _texture(new _Texture), _fixed(new _Fixed), _shader(new _Shader)
{
  DJV_DBG_OBJ2(_DBG, "Data::Data");
  
  set(in);
}

Data::~Data()
{
  DJV_DBG_OBJ2(_DBG, "Data::~Data");
}

void Data::set(const image::Data * in)
{
  DJV_DBG_OBJ2(_DBG, "Data::set");
  if (in) DJV_DBG_PRN("in = " << *in);

  _data = in;
  _texture->set(_data);
}

void Data::draw(const Options & options) const throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Data::draw");
  
  DJV_DBG_GL(glPushAttrib(
    GL_CURRENT_BIT |
    GL_ENABLE_BIT |
    GL_TRANSFORM_BIT |
    GL_PIXEL_MODE_BIT |
    GL_TEXTURE_BIT));
  DJV_DBG_GL(glDisable(GL_DITHER));
  
  Error error;
  try {
  
    if (1) {
    
      if (GLEW_ARB_fragment_shader)
        _shader->draw(options);
      else
        _fixed->draw(options);
      _texture->draw(options);
    
    } else {
    
      if (_data) {
        const image::data::Info & info = _data->info();
        DJV_DBG_GL(glDrawPixels(
          info.size.x, info.size.y,
          gl::format(info.pixel, info.bgr), gl::type(info.pixel),
          _data->data()));
      }
    }
    
  } catch (Error in) { error = in; }
  
  // XXX
  
  if (GLEW_ARB_fragment_shader) { DJV_DBG_GL(glUseProgramObjectARB(0)); }
  DJV_DBG_GL(glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE));
  
  DJV_DBG_GL(glPopAttrib());
  
  if (error.str().size()) throw error;
}

void Data::read(image::Data * out)
{
  read(out, Box2i(out->size()));
}

void Data::read(image::Data * out, const Box2i & area)
{
  DJV_DBG_OBJ2(_DBG, "Data::read");
  DJV_DBG_PRN("out = " << *out);
  DJV_DBG_PRN("area = " << area);
  
  const image::data::Info & info = out->info();
  
  DJV_DBG_GL(glPushAttrib(
    GL_CURRENT_BIT |
    GL_ENABLE_BIT |
    GL_PIXEL_MODE_BIT));
  DJV_DBG_GL(glDisable(GL_DITHER));
  
  if (GLEW_ARB_color_buffer_float) {
    DJV_DBG_GL(glClampColorARB(GL_CLAMP_READ_COLOR_ARB, GL_FALSE));
  }
  DJV_DBG_GL(glPixelStorei(GL_PACK_ALIGNMENT, info.align));
  DJV_DBG_GL(glPixelStorei(GL_PACK_SWAP_BYTES, info.endian != mem::endian()));
  DJV_DBG_GL(glPixelStorei(GL_PACK_ROW_LENGTH, info.size.x));
  DJV_DBG_GL(glPixelStorei(GL_PACK_SKIP_PIXELS, area.x));
  DJV_DBG_GL(glPixelStorei(GL_PACK_SKIP_ROWS, area.y));
  
  // XXX OpenGL sums color components for luminance.
  
  switch (pixel::format(info.pixel)) {
    case pixel::L:
    case pixel::LA:
      DJV_DBG_GL(glPixelTransferf(GL_GREEN_SCALE, 0.0));
      DJV_DBG_GL(glPixelTransferf(GL_BLUE_SCALE, 0.0));
      break;
    default: break;
  }
  
  DJV_DBG_GL(glReadPixels(
    0, 0, area.w, area.h,
    gl::format(info.pixel, info.bgr), gl::type(info.pixel),
    out->data()));
    
  DJV_DBG_GL(glPopAttrib());
  DJV_DBG_GL(glFinish());
}

}
}
}
}
