/*------------------------------------------------------------------------------
 lib/djv_image/io_ppm_data.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/io_ppm_data.cxx"

#include "io_ppm.h"

namespace djv {
namespace image {
namespace io {
namespace ppm {

size_t scanline_bytes(int width, int channels, int bit_depth, MODE mode)
{
  DJV_DBG_OBJ2(_DBG, "scanline_bytes");
  DJV_DBG_PRN("width = " << width);
  DJV_DBG_PRN("channels = " << channels);
  DJV_DBG_PRN("bit depth = " << bit_depth);
  DJV_DBG_PRN("mode = " << mode);
  
  size_t out = 0;
  
  switch (mode) {
  
    case ASCII: {
      int chars = 0;
      switch (bit_depth) {
        case  1: chars = 1; break;
        case  8: chars = 3; break;
        case 16: chars = 5; break;
        default: break;
      }
      out = (chars + 1) * width * channels + 1;
    } break;
    
    case BINARY:
      switch (bit_depth) {
        case  1: out = math::ceil(width / 8.0); break;
        case  8:
        case 16: out = width * channels; break;
        default: break;
      }
      break;
  }
  
  DJV_DBG_PRN("out = " << out);
  return out;
}

void ascii_unpack(djv::Io * io, void * out, int size, int bit_depth)
  throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "ascii_unpack");
  
  char tmp [str::cstr_size];
  int i = 0;
  
  switch (bit_depth) {
  
    case 1: {
      uint8_t * out_p = reinterpret_cast<uint8_t *>(out);
      for (; i < size; ++i) {
        djv::io::word(io, tmp, str::cstr_size);
        out_p[i] = str::str_to_int(tmp) ? 0 : 255;
      }
    } break;
    
#define _UNPACK(TYPE) \
      TYPE * out_p = reinterpret_cast<TYPE *>(out); \
      for (; i < size; ++i) { \
        djv::io::word(io, tmp, str::cstr_size); \
        out_p[i] = str::str_to_int(tmp); \
        DJV_DBG_PRN(i << " = " << out_p[i]); \
      }
    case 8: { _UNPACK(uint8_t) } break;
    case 16: { _UNPACK(uint16_t) } break;
    
    default: break;
  }
}

size_t ascii_pack(const void * in, void * out, int size, int bit_depth)
{
  char * out_p = reinterpret_cast<char *>(out);
  
  switch (bit_depth) {

    case 1: {
      const uint8_t * in_p = reinterpret_cast<const uint8_t *>(in);
      for (int i = 0; i < size; ++i) {
        out_p[0] = '0' + (! in_p[i]);
        out_p[1] = ' ';
        out_p += 2;
      }
    } break;
    
#define _PACK(TYPE) \
      const TYPE * in_p = reinterpret_cast<const TYPE *>(in); \
      for (int i = 0; i < size; ++i) { \
        const int j = str::int_to_str(in_p[i], out_p); \
        out_p[j] = ' '; \
        out_p += j + 1; \
      }
    case 8: { _PACK(uint8_t) } break;
    case 16: { _PACK(uint16_t) } break;
    
    default: break;
  }
  
  *out_p++ = '\n';
  return out_p - reinterpret_cast<char *>(out);
}

}
}
}
}
