/*------------------------------------------------------------------------------
 lib/djv_ui/app.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/app.cxx"

#include "app.h"
#include <djv_base/sys.h>
#include <djv_ui/prefs.h>
#include <djv_ui/style.h>
#include <djv_ui/icon.h>
#include <djv_ui/widget.h>
#include <djv_ui/window.h>
#include <djv_ui/dialog.h>
#include <djv_ui/file_browser.h>
#include <djv_ui/messages_dialog.h>
#include <djv_ui/prefs_dialog.h>
#include <djv_ui/about_dialog.h>
#include <djv_ui/image_io.h>
#include <FL/Fl.H>

namespace djv {
namespace ui {
namespace app {

App::App(const Str & name, int & argc, char ** argv) throw (Error) :
  djv::image::app::App(name, argc, argv),
  _valid(false)
{
  DJV_DBG_OBJ2(_DBG, "App::App");
  
  Fl::visual(FL_DOUBLE | FL_RGB);
  
  prefs::init();
  image_io::init();
}

App::~App()
{
  DJV_DBG_OBJ2(_DBG, "App::~App");

  file_browser::del();
  about_dialog::del();
  prefs_dialog::del();
  messages_dialog::del();
  dialog::del();
  icon::del();
  style::del();
  
  image_io::del();
  prefs::del();
  
  DJV_DBG_PRN("widgets = " << _widget_list.size());
  for (uint i = 0; i < _widget_list.size(); ++i)
    DJV_DBG_PRN("  " << _widget_list[i]->dbg_name());
  DJV_DBG_PRN("windows = " << _window_list.size());
}

int App::exec()
{
  DJV_DBG_OBJ2(_DBG, "App::exec");
  DJV_DBG_PRN("valid = " << _valid);
  DJV_DBG_PRN("widgets = " << _widget_list.size());
  DJV_DBG_PRN("windows = " << _window_list.size());
  
  if (djv::app::App::exit() != 0) exit();
  Fl::run();
  return djv::app::App::exit();
}

void App::exit()
{
  DJV_DBG_OBJ2(_DBG, "App::exit");
  DJV_DBG_PRN("widgets = " << _widget_list.size());
  DJV_DBG_PRN("windows = " << _window_list.size());
  
  for (uint i = 0; i < _window_list.size(); ++i)
    _window_list[i]->hide();
}

void App::valid(bool in)
{
  DJV_DBG_OBJ2(_DBG, "App::valid");
  DJV_DBG_PRN("in = " << in);
  
  _valid = in;
}

Str App::info() const
{
  return str::Format(
"%1\n"
"\n"
"UI Image Load Plugins:\n%2\n"
"\n"
"UI Image Save Plugins:\n%3"
  ).
    arg(djv::image::app::App::info()).
    arg(str::join(image_io::load()->name_list(), ", ")).
    arg(str::join(image_io::save()->name_list(), ", "));
}

void App::message(const Str & in) const
{
  if (_valid) messages_dialog::message(in);
  else djv::image::app::App::message(in);
}

void App::error(const Str & in) const { error(Error(in)); }

void App::error(const Error & in) const
{
  if (_valid) messages_dialog::error(in);
  else djv::image::app::App::error(in);
}

void App::widget_add(widget::Base * in)
{
  //DJV_DBG_OBJ2(_DBG, "App::widget_add");
  
  _widget_list += in;
}

void App::widget_del(widget::Base * in)
{
  //DJV_DBG_OBJ2(_DBG, "App::widget_del");
  
  List<widget::Base *>::iterator i = std::find(
    _widget_list.begin(), _widget_list.end(), in);
  if (i != _widget_list.end()) _widget_list.erase(i);
}

void App::window_add(window::Widget * in)
{
  DJV_DBG_OBJ2(_DBG, "App::window_add");
  
  _window_list += in;
}

void App::window_del(window::Widget * in)
{
  DJV_DBG_OBJ2(_DBG, "App::window_del");
  
  List<window::Widget *>::iterator i = std::find(
    _window_list.begin(), _window_list.end(), in);
  if (i != _window_list.end()) _window_list.erase(i);
}

void App::window_redraw()
{
  DJV_DBG_OBJ2(_DBG, "App::window_redraw");
  
  for (uint i = 0; i < _window_list.size(); ++i)
    _window_list[i]->redraw();
}

void App::window_update()
{
  DJV_DBG_OBJ2(_DBG, "App::window_update");
  
  for (uint i = 0; i < _window_list.size(); ++i)
    _window_list[i]->update();
}

App * app() { return static_cast<djv::ui::app::App *>(djv::app::app()); }

}
}
}

