/*------------------------------------------------------------------------------
 lib/djv_ui/base.h
 
 Base

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_BASE_H
#define DJV_UI_BASE_H

#include <djv_image/base.h>
#include <FL/Enumerations.H>
#include <FL/Fl_Image.H>

namespace djv {
namespace ui {

// Callbacks.

class Signal_Base;

class Callback_Base {
public:

  Callback_Base() : _callbacks(true) {}
  virtual ~Callback_Base() = 0;
  
  void del();
  
  void sig_add(Signal_Base *);
  void sig_del(Signal_Base *);
  uint sig_size() const { return _sig_list.size(); }
  
  void callbacks(bool in) { _callbacks = in; }
  bool callbacks() const { return _callbacks; }
  
private:

  List<Signal_Base *> _sig_list;
  bool _callbacks;
};

class Signal_Base {
public:

  virtual ~Signal_Base() {}
  
  virtual void release(Callback_Base *) = 0;
};

template <class T>
class Signal : public Signal_Base {
public:

  inline ~Signal();

  typedef void (Callback)(Callback_Base *, T);
  
  inline void set(Callback_Base *, Callback *);
  inline void del(Callback_Base *);
  inline void del();
  
  inline void emit(T);
  
  inline void release(Callback_Base *);

  uint size() const { return _list.size(); }

private:

  typedef std::pair<Callback_Base *, void *> Pair;
  List<Pair> _list;
};

#define DJV_CALLBACK(TYPE, NAME, ARG) \
  static void NAME(Callback_Base * that, ARG data) \
  { \
    if (that->callbacks()) \
      static_cast<TYPE *>(that)->NAME(data); \
  } \
  void NAME(ARG in)

// FLTK callbacks.

#define DJV_FL_CALLBACK(TYPE, NAME) \
  static void NAME(void * that) \
  { \
    if (static_cast<TYPE *>(that)->callbacks()) \
      static_cast<TYPE *>(that)->NAME(); \
  } \
  void NAME()
#define DJV_FL_WIDGET_CALLBACK(TYPE, NAME) \
  static void NAME(Fl_Widget *, void * that) \
  { \
    if (static_cast<TYPE *>(that)->callbacks()) \
      static_cast<TYPE *>(that)->NAME(); \
  } \
  void NAME()

// Forward declarations.

namespace prefs { class Prefs; }
namespace layout {
class Layout;
class Horizontal;
class Vertical;
class Stack;
}
namespace widget { class Base; class Widget; }
namespace window { class Widget; }
namespace group {
class Frame;
class Horizontal_Divider;
class Vertical_Divider;
class Box;
}
namespace label { class Widget; class Text; class Image; }
namespace button { class Push; class Check; class Radio; class Tool; }
namespace text { class Edit; class Display; class Multiline_Display; }
namespace choice { class Widget; class Edit; }
namespace slider { class Int; class Float; }
namespace num {
class Int_Edit; class Int_Display; class Int_Slider;
class Float_Edit; class Float_Display; class Float_Slider;
}
namespace columns { class Widget; }
namespace file { class Widget; }
namespace tab { class Widget; }
namespace menu { class Widget; class Popup; }
namespace dialog { class Message; class Question; class Input; class Choice; }
namespace file_browser { class Dialog; }
namespace prefs_dialog { class Dialog; }
namespace gl { class Widget; }
namespace pixel { class Widget; }
namespace color { class Swatch; class Widget; }
namespace image { class Widget; }
namespace image_io {
class Load;
class Save;
class Load_Factory;
class Save_Factory;
}

// Cursor.

class Cursor {
public:
  Cursor(const Fl_Cursor &);
  ~Cursor();
};

void cursor(const Fl_Cursor &);

// Functions.

Vec2i str_size(const Str &);
int str_width(const Str &);

void recent(const Str &, List<Str> *, int max = 10);

// Conversion.

Fl_Image * image_to_fl(const djv::image::Data &);

}
}

#include <djv_ui/base_.h>

#endif

