/*------------------------------------------------------------------------------
 lib/djv_ui/button_check.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "button.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Check_Button.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace button {

class Check::Widget : public Fl_Check_Button, public widget::Base {
public:

  Widget() : Fl_Check_Button(0, 0, 0, 0) {}

  int handle(int in)
  {
    switch (widget::Base::handle(in)) {
      case -1: return 0;
      case 1: return 1;
    }
    switch (in) {
  
      // XXX Override FLTK behavior.
    
      case FL_SHORTCUT:
        if (! (shortcut() ? Fl::test_shortcut(shortcut()) : test_shortcut()))
          return 0;
        if (visible_focus() && handle(FL_FOCUS))
          Fl::focus(this);
        value(! value());
        return 1;
    }
    return Fl_Check_Button::handle(in);
  }
  
  void draw()
  {
    Fl_Check_Button::draw();
    widget::Base::draw();
  }

  void dirty()
  {
    widget::Base::dirty();

    labelfont(text_font());
    labelsize(style::style()->font_size());
    labelcolor(text_color());
  
    fl_font(text_font(), style::style()->font_size());
    _size_hint = Vec2i(fl_height()) + style::style()->margin_widget() * 2;
  }
  
  void resize(int x, int y, int w, int h)
  {
    Fl_Check_Button::resize(x, y, w, h);
    layout::Item::resize(x, y, w, h);
  }
  
  Vec2i size_hint() const { return _size_hint; }
  int visible() const { return Fl_Check_Button::visible(); }
  
private:

  Vec2i _size_hint;
};

Check::Check(const Str & in) :
  _value(false)
{
  _widget = new Widget;
  Fl_Group::add(_widget);

  //text_font(style::style()->font_bold());

  size_policy(FIXED, FIXED);
  align_policy(LEFT, CENTER);
  
  label(in);
  
  _widget->callback(widget_callback, this);
}
  
void Check::set(bool in)
{
  if (in == _value) return;
  _value = in;
  _widget->value(_value);
  sig.emit(_value);
}

void Check::label(const Str & in)
{
  if (in == _label) return;
  _label = in;
  _label_tmp = " " + _label; // XXX Add spacing.
  
  const int size = _label_tmp.size() + 1;
  _label_ascii.size(size);
  str::ascii(_label_tmp, _label_ascii(), size);
  _widget->label(_label_ascii());
}

void Check::dirty()
{
  widget::Widget::dirty();
  
  _widget->labelfont(text_font());
  _widget->labelsize(style::style()->font_size());
  _widget->labelcolor(text_color());
  
  fl_font(text_font(), style::style()->font_size());
  const Vec2i label_size = str_size(_label_ascii());

  const int margin = style::style()->margin_widget();
  _size_hint = Vec2i(
    fl_height() + margin + label_size.x,
    label_size.y + margin * 2);
}

void Check::resize(int x, int y, int w, int h)
{
  widget::Widget::resize(x, y, w, h);
  _widget->resize(x, y, w, h);
}

void Check::widget_callback() { set(_widget->value() ? true : false); }

}
}
}
