/*------------------------------------------------------------------------------
 lib/djv_ui/choice_edit.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "choice.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/text.h>
#include <djv_ui/menu.h>
#include <FL/Fl.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace choice {

Edit::Edit(const List<Str> & list) :
  _list(list)
{
  frame(FL_DOWN_FRAME);
  
  // Widgets.
  
  _widget = new text::Edit;
  _widget->frame(FL_NO_BOX);
  
  _menu = new menu::Popup;

  // Layout.

  size_policy(FIXED, FIXED);
  align_policy(LEFT, CENTER);
  
  layout::Horizontal * layout = new layout::Horizontal(this);
  layout->spacing(2);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  layout->add(_menu);
  
  // Initialize.

  list_update();

  // Callbacks.

  _widget->sig.set(this, widget_callback);
  _menu->sig.set(this, menu_callback);
}

void Edit::set(const Str & in)
{
  _widget->set(in);
}
  
Str Edit::get() const { return _widget->get(); }

void Edit::list(const List<Str> & in)
{
  if (in == _list) return;
  _list = in;
  list_update();
}

void Edit::widget_del()
{
  _widget->widget_del();
  Frame::widget_del();
}

void Edit::size_str(const Str & in)
{
  _widget->size_str(in);
}

void Edit::widget_callback(const Str & in)
{
  sig.emit(in);
}

void Edit::menu_callback(int in)
{
  set(_list[in]);
}

void Edit::list_update()
{
  _menu->set(_list);
  
  // Set size to width of largest menu item.
  
  if (_list.size() && ! _widget->size_str().size()) {
  
    fl_font(text_font(), style::style()->font_size());
    
    uint i = 0;
    for (uint j = 1; j < _list.size(); ++j)
      if (fl_width(_list[j].c_str()) > fl_width(_list[i].c_str()))
        i = j;
    _widget->size_str(_list[i] + " ");
    
    update();
  }
}

}
}
}
