/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser_item.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/file_browser_item.cxx"

#include "_file_browser.h"
#include <djv_image/image.h>
#include <djv_ui/app.h>

namespace djv {
namespace ui {
namespace file_browser {

Item::Item() :
  prev(0), next(0),
  _odd(false), _height(0),
  _image(0), _image_valid(false), _image_init(false)
{}

Item::~Item()
{
  if (_image) delete _image;
}

void Item::set(const File & in, bool odd)
{
  _file = in;
  _odd = odd;
  
  _image_valid = djv::image::io::valid(_file);
}

namespace {

Vec2i thumbnail_size(const Vec2i & in, int size, int * proxy)
{
  const int image_size = math::max(in.x, in.y);
  if (image_size <= 0) return Vec2i();

  int _proxy = 0;
  double proxy_scale = double(djv::image::data::proxy_scale(_proxy));
  while ((image_size / proxy_scale) > size * 2)
    proxy_scale = double(djv::image::data::proxy_scale(++_proxy));
  if (proxy) *proxy = _proxy;
  
  double scale = size / double(image_size / proxy_scale);
  return Vec2i(
    math::ceil(in.x / proxy_scale * scale),
    math::ceil(in.y / proxy_scale * scale));
}

}

Fl_Image * Item::image(int thumbnail) const
{
  if (_image_init) return _image;
  
  DJV_DBG_OBJ2(_DBG, "Item::image");
  DJV_DBG_PRN("file = " << _file);
  DJV_DBG_PRN("thumbnail = " << thumbnail);
  
  Item * that = const_cast<Item *>(this);
  that->_image_init = true;

  if (djv::image::io::valid(_file)) {
    ui::Cursor cursor(FL_CURSOR_WAIT);
  try {
    std::auto_ptr<djv::image::io::Load> load(
      djv::image::io::load(_file, &that->_image_io_info));
    DJV_DBG_PRN("info = " << _image_io_info);

    int proxy = 0;
    djv::image::Image tmp(djv::image::Info(
      thumbnail_size(_image_io_info.size, thumbnail, &proxy),
      djv::image::pixel::RGB_U8));
    DJV_DBG_PRN("tmp = " << tmp);

    const djv::image::Image * image = load->get(-1, proxy);
    DJV_DBG_PRN("image = " << *image);

    djv::image::gl::data::Options options;
    options.scale = tmp.info().size;
    options.profile = image->info().profile;
    djv::image::gl::data::copy(*image, &tmp, options);
    that->_image = image_to_fl(tmp);
  
  } catch (Error in) {
    app::app()->error(str::Format("%1: %2").arg(in).arg(_file.name()));
  }
  }

  return _image;
}

}
}
}
