/*------------------------------------------------------------------------------
 lib/djv_ui/image_fnc.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "image.h"

namespace djv {
namespace ui {
namespace image {

double scale(SCALE in)
{
  static double data [] = { 1.0, 16.0 / 9.0, 1.85, 2.35 };
  return data[in];
}

double rotate(ROTATE in)
{
  static double data [] = { 0.0, 90.0, 180.0, 270.0 };
  return data[in];
}

Vec2i size(const Vec2i & in, SCALE scale, ROTATE rotate)
{
  Vec2i out = in;
  switch (scale) {
    case SCALE_NONE: break;
    default: out.x = int(in.y * image::scale(scale));
  }
  switch (rotate) {
    case ROTATE_90:
    case ROTATE_270: out = swap(out); break;
    default: break;
  }
  return out;
}

Vec2i position(const Vec2i & in, const Vec2i & size, ROTATE rotate)
{
  Vec2i out = in;
  switch (rotate) {
    case ROTATE_90: out += Vec2i(0, size.x); break;
    case ROTATE_180: out += size; break;
    case ROTATE_270: out += Vec2i(size.y, 0); break;
    default: break;
  }
  return out;
}
 
const List<Str> & scale_label()
{
  static const List<Str> data = List<Str>() << "None" << "16:9" << "1.85" <<
    "2.35";
  return data;
}

const List<Str> & rotate_label()
{
  static const List<Str> data = List<Str>() << "0" << "90" << "180" << "270";
  return data;
}

}
}
}
