/*------------------------------------------------------------------------------
 lib/djv_ui/menu.h
 
 Menu Widgets

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_MENU_H
#define DJV_UI_MENU_H

#include <djv_ui/group.h>

class Fl_Menu_Item;

namespace djv {
namespace ui {
namespace menu {

// Menu widget base.

class Base {
public:

  Base();
  virtual ~Base();
  
  virtual int menu_add(const Str & label, int shortcut = 0,
    Fl_Callback * = 0, void * = 0, int flags = 0);
  virtual void menu_end();
  virtual void menu_del();

protected:

  virtual void menu(Fl_Menu_Item *) = 0;

private:

  class Item;
  
  List<Item *> _list;
  int _depth;
};

// Menu widget.

enum { TOGGLE = 2 << 0, RADIO = 2 << 1, VALUE = 2 << 2, SUB_MENU = 2 << 3,
  DIVIDER = 2 << 4 };

class Widget : public group::Frame, public Base {
public:

  Widget();
  
  // Value.

  int menu() const;
  bool value() const;
  
  // Layout.

  virtual void dirty();
  virtual void resize(int, int, int, int);
  virtual Vec2i size_hint() const { return _size_hint; }

private:

  class _Widget;

  void menu(Fl_Menu_Item *);
  
  _Widget * _widget;
  Vec2i _size_hint;
};

// Menu popup widget.

class Popup : public group::Frame, public Base {
public:

  Popup(const List<Str> & = List<Str>());
  
  // Value.
  
  void set(const List<Str> &);
  void add(const List<Str> &);
  Signal<int> sig;
  
  // Widget.
  
  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void resize(int, int, int, int);
  virtual Vec2i size_hint() const { return _size_hint; }
  
private:

  class Widget;
  
  DJV_FL_WIDGET_CALLBACK(Popup, widget_callback);

  void menu(Fl_Menu_Item *);

  Widget * _widget;
  Vec2i _size_hint;
  Fl_Image * _icon;
};

// Functions.

Str list_name(const Str &, int index);

}
}
}

#endif
