/*------------------------------------------------------------------------------
 plugin/djv_exr/ui.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_exr/ui.cxx"

#include "ui.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/num_float.h>

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_load()
  { return new Load_Widget; }
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_save()
  { return new Save_Widget; }
}

Load_Widget::Load_Widget() :
  _plugin(0)
{
  DJV_DBG_OBJ2(_DBG, "Load_Widget::Load_Widget");

  // Widgets.
  
  ui::group::Box * color_group = new ui::group::Box;
  _color_widget = new ui::choice::Widget(color_label());
  _color_widget->label(new ui::label::Widget("Color:"));
  color_group->label_widget(_color_widget);
  ui::label::Text * color_text = new ui::label::Text("Input color space.");
  
  ui::group::Box * exposure_group = new ui::group::Box("Exposure");
  _exposure_widget = new ui::num::Float_Slider;
  _exposure_widget->default_value(::Load::Options().exposure.value);
  _exposure_widget->range(-10.0, 10.0);
  _exposure_widget->inc(0.1, 1.0);
  _exposure_widget->label(new ui::label::Widget("Exposure:"));
  _defog_widget = new ui::num::Float_Slider;
  _defog_widget->default_value(::Load::Options().exposure.defog);
  _defog_widget->range(0.0, 0.01);
  _defog_widget->inc(0.0001, 0.001);
  _defog_widget->label(new ui::label::Widget("Defog:"));
  _knee_low_widget = new ui::num::Float_Slider;
  _knee_low_widget->default_value(::Load::Options().exposure.knee_low);
  _knee_low_widget->range(-3.0, 3.0);
  _knee_low_widget->inc(0.1, 1.0);
  _knee_low_widget->label(new ui::label::Widget("Knee Low:"));
  _knee_high_widget = new ui::num::Float_Slider;
  _knee_high_widget->default_value(::Load::Options().exposure.knee_high);
  _knee_high_widget->range(3.5, 7.5);
  _knee_high_widget->inc(0.1, 1.0);
  _knee_high_widget->label(new ui::label::Widget("Knee High:"));
  _gamma_widget = new ui::num::Float_Slider;
  _gamma_widget->default_value(::Load::Options().exposure.gamma);
  _gamma_widget->range(0.1, 4.0);
  _gamma_widget->label(new ui::label::Widget("Gamma:"));
  
  // Layout.

  ui::layout::group(List<ui::layout::Item *>() << _exposure_widget <<
    _defog_widget << _knee_low_widget << _knee_high_widget << _gamma_widget);
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());

  layout->add(color_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(color_group);
  group_layout->add(color_text);

  layout->add(exposure_group);  
  group_layout = new ui::layout::Vertical(exposure_group);
  group_layout->add(_exposure_widget);
  group_layout->add(_defog_widget);
  group_layout->add(_knee_low_widget);
  group_layout->add(_knee_high_widget);
  group_layout->add(_gamma_widget);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _color_widget->sig.set(this, color_callback);
  _exposure_widget->sig.set(this, exposure_callback);
  _defog_widget->sig.set(this, defog_callback);
  _knee_low_widget->sig.set(this, knee_low_callback);
  _knee_high_widget->sig.set(this, knee_high_callback);
  _gamma_widget->sig.set(this, gamma_callback);
}

void Load_Widget::set(image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Load_Widget::color_callback(int in)
{
  if (! _plugin) return;
  _options.color = COLOR(in);
  plugin_update();
  sig.emit(_plugin);
}

void Load_Widget::exposure_callback(double in)
{
  if (! _plugin) return;
  _options.exposure.value = in;
  plugin_update();
  sig.emit(_plugin);
}

void Load_Widget::defog_callback(double in)
{
  if (! _plugin) return;
  _options.exposure.defog = in;
  plugin_update();
  sig.emit(_plugin);
}

void Load_Widget::knee_low_callback(double in)
{
  if (! _plugin) return;
  _options.exposure.knee_low = in;
  plugin_update();
  sig.emit(_plugin);
}

void Load_Widget::knee_high_callback(double in)
{
  if (! _plugin) return;
  _options.exposure.knee_high = in;
  plugin_update();
  sig.emit(_plugin);
}

void Load_Widget::gamma_callback(double in)
{
  if (! _plugin) return;
  _options.exposure.gamma = in;
  plugin_update();
  sig.emit(_plugin);
}

void Load_Widget::plugin_update()
{
  callbacks(false);

  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_options.color, color_label());
    _plugin->option(_plugin->option_list()[::Load::OPTION_COLOR], &tmp);
    tmp = serialize(_options.exposure);
    _plugin->option(_plugin->option_list()[::Load::OPTION_EXPOSURE], &tmp);
  }

  callbacks(true);
}

void Load_Widget::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[::Load::OPTION_COLOR]);
    str::serialize(&tmp, &_options.color, color_label());
    tmp = _plugin->option(_plugin->option_list()[::Load::OPTION_EXPOSURE]);
    serialize(&tmp, &_options.exposure);
  }
  
  _color_widget->set(_options.color);
  _exposure_widget->set(_options.exposure.value);
  _defog_widget->set(_options.exposure.defog);
  _knee_low_widget->set(_options.exposure.knee_low);
  _knee_high_widget->set(_options.exposure.knee_high);
  _gamma_widget->set(_options.exposure.gamma);
  
  callbacks(true);
}

Save_Widget::Save_Widget() :
  _plugin(0)
{
  DJV_DBG_OBJ2(_DBG, "Save_Widget::Save_Widget");
  
  // Widgets.
  
  ui::group::Box * compress_group = new ui::group::Box;
  _compress_widget = new ui::choice::Widget(::compress_label());
  _compress_widget->label(new ui::label::Widget("Compress:"));
  compress_group->label_widget(_compress_widget);
  ui::label::Text * compress_text = new ui::label::Text("File compression.");
  
  // Layout.

  ui::layout::group(List<ui::layout::Item *>() << _compress_widget);
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());

  layout->add(compress_group);
  ui::layout::Vertical * group_layout =
    new ui::layout::Vertical(compress_group);
  group_layout->add(compress_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _compress_widget->sig.set(this, compress_callback);
}

void Save_Widget::set(image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Save_Widget::compress_callback(int in)
{
  if (! _plugin) return;
  _options.compress = COMPRESS(in);
  plugin_update();
  sig.emit(_plugin);
}

void Save_Widget::plugin_update()
{
  callbacks(false);

  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_options.compress, compress_label());
    _plugin->option(_plugin->option_list()[::Save::OPTION_COMPRESS], &tmp);
  }

  callbacks(true);
}

void Save_Widget::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[::Save::OPTION_COMPRESS]);
    str::serialize(&tmp, &_options.compress, compress_label());
  }
  
  _compress_widget->set(_options.compress);
  
  callbacks(true);
}
