/*------------------------------------------------------------------------------
 plugin/djv_tiff/plugin.h

 Tagged Image File Format (TIFF) I/O

 Requires: 
 * libtiff - http://www.libtiff.org
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_TIFF_PLUGIN_H
#define DJV_TIFF_PLUGIN_H

#include <djv_image/io.h>
#include <tiffio.h>

using namespace djv;

// Information.

static const Str plugin_name = "TIFF";
static const List<Str> extension_list = List<Str>() << ".tiff" << ".tif";

// Options.

enum COMPRESS { COMPRESS_NONE, COMPRESS_RLE, COMPRESS_LZW };

// Load.

class Load : public image::io::Load {
public:

  Load();
  ~Load();

  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  plugin::Plugin * copy() const { return new Load; }
  
  void open(const File &, image::io::Info *) throw (Error);
  const image::Image * get(int frame, int proxy) throw (Error);

private:

  void _open(const File &, image::io::Info *) throw (Error);
  void _close();

  File _file;
  TIFF * _f;
  bool _planar, _compress, _palette;
  uint16 * _colormap [3];
  image::Image _image, _image_tmp;
};

// Save.

class Save : public image::io::Save {
public:

  Save();
  ~Save();
  
  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  Str help() const;
  
  plugin::Plugin * copy() const;
  
  struct Options {
    Options() : compress(COMPRESS_NONE) {}
    COMPRESS compress;
  };
  enum OPTION { OPTION_COMPRESS };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const image::io::Info &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:

  void _open(const File &, const image::tag::Tag &) throw (Error);
  void _close();

  Options _options;
  File _file;
  TIFF * _f;
  image::Image _image;
};

// Functions.

void tiff_init();

void palette_unpack(
  uint8_t *, int size, int bytes,
  uint16_t * red, uint16_t * green, uint16_t * blue
);

// Labels.

const List<Str> & compress_label();
const List<Str> & save_option_label();

#endif

