/*------------------------------------------------------------------------------
 plugin/djv_vlut/plugin.h
 
 Video Lookup Table (VLUT) Image I/O
 
 Requires: 
 * X11 Video Mode Extension - http:://www.x.org
 * X11 Video Control Extension - http:://www.sgi.org/irix

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VLUT_PLUGIN_H
#define DJV_VLUT_PLUGIN_H

#include <djv_image/io.h>
#if defined(DJV_VLUT_XORG)
#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>
#elif defined(DJV_VLUT_XSGI)
#include <X11/Xlib.h>
#include <X11/extensions/XSGIvc.h>
#elif defined(DJV_VLUT_WIN32)
#include <windows.h>
//#include <wingdi.h>
#endif

using namespace djv;

// Information.

static const Str plugin_name = "VLUT";

// Video LUT handle.

class Handle {
public:

  Handle(const Str & name) throw (Error);
  ~Handle();

# if defined(DJV_VLUT_XORG) || defined(DJV_VLUT_XSGI)
  Display * display;
  int screen;
# elif defined(DJV_VLUT_WIN32)
  HDC display;
# endif
};

// Load.

class Load : public image::io::Load {
public:
  
  Str name() const { return ::plugin_name; }
  
  plugin::Plugin * copy() const { return new Load; }
  
  void open(const File &, image::io::Info *) throw (Error);
  const image::Image * get(int frame, int proxy) throw (Error);

private:

  Str _vlut;
  std::auto_ptr<Handle> _f;
  image::io::Info _info;
  image::Image _image, _image_tmp;
};

// Save.

class Save : public image::io::Save {
public:
  
  Str name() const { return ::plugin_name; }
  
  plugin::Plugin * copy() const { return new Save; }
  
  void open(const File &, const image::io::Info &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:

  Str _vlut;
  std::auto_ptr<Handle> _f;
  int _size;
  image::Image _image, _image_tmp;
};

#endif

