/*------------------------------------------------------------------------------
 plugin/djv_vlut/save.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_vlut/save.cxx"

#include "plugin.h"

using namespace image;

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_save() { return new Save; }
}

void Save::open(const File & in, const image::io::Info &) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::open");
  DJV_DBG_PRN("in = " << in);

  // Open.
    
  _vlut = in;
  _f = std::auto_ptr<Handle>(new Handle(in));

  // Information.
  
#if defined(DJV_VLUT_XORG)

  if (! XF86VidModeGetGammaRampSize(_f->display, _f->screen, &_size))
    throw Error(plugin_name, image::io::error_open);

#elif defined(DJV_VLUT_XSGI)

  int maps = 0, precision = 0, attr = 0;
  if (! XSGIvcQueryScreenGammaMaps(_f->display, _f->screen, &maps))
    throw Error(plugin_name, image::io::error_open);
  if (maps <= 0)
    throw Error(plugin_name, image::io::error_open);
  if (! XSGIvcQueryGammaMap(_f->display, _f->screen, 0,
    &_size, &precision, &attr)
  )
    throw Error(plugin_name, image::io::error_open);
  if (attr & XSGIVC_GMWriteLock)
    throw Error(plugin_name, image::io::error_open);

#elif defined(DJV_VLUT_WIN32)

  _size = 256;

#endif

  DJV_DBG_PRN("vlut size = " << _size);
  
  // Temporary buffers.
  
  Info info;
  info.size = Vec2i(_size, 1);
  info.pixel = pixel::RGB_U16;
  
  _image.set(info);
  _image_tmp.set(info);
}

void Save::set(const Image & in, int) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::set");
  DJV_DBG_PRN("in = " << in);
  
  // Temporary buffers.
  
  const Image * image_p = &in;
  if (in.info() != _image.info()) {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    gl::data::Options options;
    options.scale = _image.size();
    gl::data::copy(in, &_image, options);
    image_p = &_image;
  }
  
  data::planar_deinterleave(*image_p, &_image_tmp);
  pixel::U16_T * p = reinterpret_cast<pixel::U16_T *>(_image_tmp.data());
  
#if defined(DJV_DBG)
  //for (int i = 0; i < _size; ++i) DJV_DBG_PRN(i << " = " << p[i]);
#endif

  // Write.
  
#if defined(DJV_VLUT_XORG)

  if (! XF86VidModeSetGammaRamp(_f->display, _f->screen, _size, p, p + _size,
    p + _size * 2)
  )
    throw Error(plugin_name, image::io::error_write);

#elif defined(DJV_VLUT_XSGI)

  XSGIvcStoreGammaColors16(_f->display, _f->screen, 0, _size,
    XSGIVC_MComponentRed, p);
  XSGIvcStoreGammaColors16(_f->display, _f->screen, 0, _size,
    XSGIVC_MComponentGreen, p + _size);
  XSGIvcStoreGammaColors16(_f->display, _f->screen, 0, _size,
    XSGIVC_MComponentBlue, p + _size * 2);

#elif defined(DJV_VLUT_WIN32)

  ::SetDeviceGammaRamp(_f->display, p);

#endif
}

