//#define DJV_DBG

#include <djv_base/file.h>
#include <djv_image/io.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/image.h>
#include <djv_ui/app.h>
#include <FL/Fl.H>

using namespace djv;

class App : public ui::app::App {
public:

  App(int argc, char ** argv);
  ~App();
  
  void update();

private:

  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  File _file;
  image::io::Load * _load;
  image::io::Info _info;
  image::gl::data::Data _data;
  ui::window::Widget * _window;
  ui::image::Widget * _widget;
  int _frame;
};

void update_callback(void *);

App::App(int argc, char ** argv) :
  ui::app::App("image_play", argc, argv),
  _load(0),
  _window(0),
  _frame(0)
{
  if (argc != 2) {
    print("Usage: image_play (input)");
    djv::app::App::exit(1);
    return;
  }
  
  _file.set(argv[1]);
  if (_file.seq_valid()) _file.type = file::SEQ;
  try { _load = image::io::load(_file, &_info); }
  catch (Error in) {
    error(str::Format("%1: %2").arg(_file).arg(in));
    djv::app::App::exit(1);
    return;
  }
  
  // Widgets.
  
  _window = new ui::window::Widget(name());
  _widget = new ui::image::Widget;
  _widget->set(&_data);
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(_window);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  
  // Initialize.
  
  _frame = _info.time.start;
  //update();
  Fl::add_idle(update_callback, this);

  _window->size(_info.size);
  _window->show();

  // Callbacks.
  
  _window->close_sig.set(this, exit_callback);

  valid(true);
}

App::~App()
{
  if (_window) _window->widget_del();
  if (_load) delete _load;
}

void App::update()
{
  DJV_DBG_OBJ("App::update");

  static djv::time::Timer t;
  t.check();
  app::App::print(str::Format("FPS = %1").arg(t.fps()));
  t.start();
  
  /*for (int i = 0; i < 3; ++i) {
    try { djv::io::Io io; io.open(_file.get(_frame + i), djv::io::READ); }
    catch (Error) {}
  }*/

  const image::Image * image = 0;
  try { image = _load->get(_frame); }
  catch (Error) {}
  if (image) {
    _data.set(image);
    _widget->profile(image->info().profile);
    _widget->redraw();
  }
  
  ++_frame;
  if (_frame > _info.time.end) _frame = _info.time.start;
}

void update_callback(void * in) { reinterpret_cast<App *>(in)->update(); }

int main(int argc, char ** argv) { return App(argc, argv).exec(); }

